/** 
 * bleifrei utility library
 *
 * Copyright (C) 2003 Ulf Borgenstam, Jonas Svensson.
 *
 * Do not distribute. If you know someone interested in obtaining these sources
 * please have them contact us.
 */

// includes
#include <fstream>

#include "../io/console.hpp"

#include "MeshManager.hpp"

using namespace bleifrei::io;
using namespace std;

namespace bleifrei {
	namespace mesh {

		MeshManager::MeshManager(const string &directory)
		{
			this->directory = directory;
		}

		MeshManager::~MeshManager(void)
		{
			map<string, Mesh *>::iterator it;
			for (it = meshes.begin(); it != meshes.end(); it++) {
				delete it->second;
			}
		}

		Mesh *MeshManager::load(const string &filename, bool calculateEdges)
		{
			Mesh *mesh;

			map<string, Mesh *>::iterator it = meshes.find(filename);

			if (it != meshes.end()) return it->second;

			try {
				mesh = new Mesh(ifstream((directory + filename).c_str(), ios::binary), calculateEdges);		
				meshes.insert(map<string, Mesh *>::value_type(filename, mesh));
			}
			catch (file_format_exception) {
				console::printf(1, 0, 0, "Error(\"%s\"): Unknown file format!", filename.c_str());
				throw;
			}
			catch (unknown_version_exception) {
				console::printf(1, 0, 0, "Error(\"%s\"): Unknown mesh version!", filename.c_str());
				throw;
			}
			catch (load_failed_exception) {
				console::printf(1, 0, 0, "Error(\"%s\"): Could not load mesh!", filename.c_str());
				throw;
			}

			return mesh;
		}

	}
}